<?php
namespace raas\apps\account\isp;


function create($data)
{
	global $nidb;

	$filter = array(
		'isp_id'	=> new \FilterVarID(true),
		'own_id'	=> new \FilterVarID(true),
		'acct_id'	=> new \FilterVarID(true),
		'acct'		=> new \FilterVarStr(true),
		'pass'		=> new \FilterVarStr(true),
		'vlan_in'	=> new \FilterVarInt(true),
		'vlan_out'	=> new \FilterVarInt(true)
	);
	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}

	$sql = "select `isp_id`, `name` from cloud_enterpise_isp where `isp_id` = ?";
	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $frmData['isp_id'], \PDO::PARAM_INT);
		if($sth->execute())
			$row = $sth->fetch(\PDO::FETCH_ASSOC);
		else
			$row = false;
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$row) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '找不到指定的接入运营商！');
		return false;
	}

	$frmData['update_time'] = time();
	$frmData['create_time'] = $frmData['update_time'];

	$nidb->beginTransaction();

	try {
		$sql = "insert into `cloud_account_isp` (`acct_id`,`isp_id`,`own_id`, `acct`, `pass`, `vlan_in`, `vlan_out`, `update_time`, `create_time`)";
		$sql.= " values (?,?,?,?,?,?,?,?,?)";
		$sql.= " on duplicate key update";
		$sql.= " `pass` = ?,`vlan_in` = ?,`vlan_out` = ?";
		$sql.= ",`update_time` = ?";

		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $frmData['acct_id'], \PDO::PARAM_INT);
		$sth->bindParam(2, $frmData['isp_id'], \PDO::PARAM_INT);
		$sth->bindParam(3, $frmData['own_id'], \PDO::PARAM_INT);
		$sth->bindParam(4, $frmData['acct'], \PDO::PARAM_STR);
		$sth->bindParam(5, $frmData['pass'], \PDO::PARAM_STR);
		$sth->bindParam(6, $frmData['vlan_in'], \PDO::PARAM_INT);
		$sth->bindParam(7, $frmData['vlan_out'], \PDO::PARAM_INT);
		$sth->bindParam(8, $frmData['update_time'], \PDO::PARAM_INT);
		$sth->bindParam(9, $frmData['create_time'], \PDO::PARAM_INT);
		
		$sth->bindParam(10, $frmData['pass'], \PDO::PARAM_STR);
		$sth->bindParam(11, $frmData['vlan_in'], \PDO::PARAM_INT);
		$sth->bindParam(12, $frmData['vlan_out'], \PDO::PARAM_INT);
		$sth->bindParam(13, $frmData['update_time'], \PDO::PARAM_INT);
		$sth->execute();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	$nidb->commit();

	return $frmData;
}

function select($data)
{
	global $nidb;
	global $user;

	$values = array();
	$optional = array();
	// set order, limit, offset value
	format_list_arg($data, $optional);

	$where_str = '';
	if(format_and_push($data, 'disabled', $optional, '', 'int', false))
		$where_str .= "`disabled` = " . $optional['disabled'] . " and ";
	else
		$where_str .= "`disabled` < " . TB_ROW_DEL_VALUE . " and ";

	if(format_and_push($data, 'upstartm', $optional, '', 'int', false))
		$where_str .= "`update_time` >= " . $optional['upstartm'] . " and ";

	if(format_and_push($data, 'endtm', $optional, '', 'int', false)) 
		$where_str .= "`update_time` < " . $optional['upendtm']  . " and ";

	if(format_and_push($data, 'startm', $optional, '', 'int', false))
		$where_str .= "`create_time` >= " . $optional['startm'] . " and ";

	if(format_and_push($data, 'endtm', $optional, '', 'int', false))
		$where_str .= "`create_time` < " . $optional['endtm']  . " and ";

	// set custom options
	if(format_and_push($data, 'acct_id', $optional, '', 'int', false))
		$where_str .= "`acct_id` = " . $optional['acct_id']  . " and ";

	if(format_and_push($data, 'isp_id', $optional, '', 'int', false))
		$where_str .= "`isp_id` = " . $optional['isp_id']  . " and ";

	if(format_and_push($data, 'ids', $optional, '', 'string', false)) {
		$idsstr = explode(',', $optional['ids']);
		$ids = array();
		foreach($idsstr as $val) {
			$val = intval($val);
			if($val > 0)
				array_push($ids, $val);
		}
		if(count($ids) > 0)
			$where_str .= "`isp_id` in (" . implode(',', $ids) . ") and ";
	}
	
	if(format_and_push($data, 'own_id', $optional, '', 'int', false))
		$where_str .= "`own_id` = " . $optional['own_id']  . " and ";

	if(format_and_push($data, 'acct', $optional, '', 'string', false)) {
		$where_str .= "`acct` = ? and ";
		array_push($values, $optional['acct']);
	}

	if(isset($data['search'])) {
		if(format_and_push($data['search'], 'value', $optional, 'search', 'string', false)) {
			$where_str .= "(`acct` like ?) and ";
			array_push($values, "%" . $optional['search'] . "%");
		}
	}

	$order_map = array(
		"isp_id",
		"acct_id",
		"acct",
		"vlan_in",
		"vlan_out",
		"disabled",
		"update_time",
		"create_time"
	);

	$order = array();
	if(isset($optional['order'])) {
		foreach($optional['order'] as $op) {
			if(isset($op['column']) && isset($data['columns'][$op['column']]['data'])) {
				if(in_array($data['columns'][$op['column']]['data'], $order_map))
					array_push($order, "`" . $data['columns'][$op['column']]['data'] . "` " . $op['dir']);
			}
		}
	}

	if(count($order) > 0)
		$order_by = 'order by ' . implode(', ', $order);
	else
		$order_by = '';
/*
	// 限制设置了负责企业的操作员只能访问其负责的企业
	if(gettype($user) == 'object' && $user->entcnt > 0) {
		$sql = "select group_concat(`ent_id`) as `ids` from cloud_user_group_list where `user_id` = ? group by `ent_id`";
		try {
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $user->user_id, \PDO::PARAM_INT);
			if($sth->execute())
				$row = $sth->fetch(\PDO::FETCH_ASSOC);
			else
				$row = false;
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}

		if($row && empty($row['ids']) == false) {
			$where_str .= '(`ent_id` in (' . $row['ids'] . ') or `admin` = ?) and ';
			array_push($values, $user->account);
		}
	}
*/
	if($where_str != '')
		$where_str = substr("where $where_str", 0, -5);

	$sql = "select count(*) as `total` from cloud_account_isp $where_str";
	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);

		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	if(count($rows) > 0) {
		$result = array(
			'total' => $rows[0]->total,
		);
	}
	else {
		$result = array(
			'total' => 0,
		);
	}

	$keys = '`isp_id`,
`acct_id`,
`acct`,
`pass`,
`vlan_in`,
`vlan_out`,
`disabled`,
`update_time`,
`create_time`';

	$sql = "select $keys from cloud_account_isp $where_str $order_by limit ? offset ? ";

	array_push($values, $optional['limit']);
	array_push($values, $optional['offset']);

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	$result['rows'] = $rows;
//	$result['sql'] = $sql;
//	$result['values'] = $values;
//  select `ent_id`, `name`, `domain`, `iwancnt`, `iwacctcnt`, `province`, `city`, `district`, `address`, `remark` from cloud_account_isp where `disabled` < 120 and `iwansvr` = 1 order by `acctcnt` desc limit 20 offset 0 

	if(isset($data['draw'])) $result['draw'] = $data['draw'];

	return $result;
}

function api_select($data)
{
	global $nidb, $user;

	$values = array();
	$optional = array();
	// set order, limit, offset value
//	format_list_arg($data, $optional);

	$where_str = '';
	if(format_and_push($data, 'disabled', $optional, '', 'int', false))
		$where_str .= "`acct`.`disabled` = " . $optional['disabled'] . " and ";
	else
		$where_str .= "`acct`.`disabled` < " . TB_ROW_DEL_VALUE . " and ";

	if(format_and_push($data, 'upstartm', $optional, '', 'int', false))
		$where_str .= "`acct`.`update_time` >= " . $optional['upstartm'] . " and ";

	if(format_and_push($data, 'endtm', $optional, '', 'int', false)) 
		$where_str .= "`acct`.`update_time` < " . $optional['upendtm']  . " and ";

	if(format_and_push($data, 'startm', $optional, '', 'int', false))
		$where_str .= "`acct`.`create_time` >= " . $optional['startm'] . " and ";

	if(format_and_push($data, 'endtm', $optional, '', 'int', false))
		$where_str .= "`acct`.`create_time` < " . $optional['endtm']  . " and ";

	// set custom options
	if(format_and_push($data, 'acct_id', $optional, '', 'int', false))
		$where_str .= "`acct`.`acct_id` = " . $optional['acct_id'] . " and ";

	if(format_and_push($data, 'isp_id', $optional, '', 'int', false))
		$where_str .= "`acct`.`isp_id` = " . $optional['isp_id']  . " and ";

	if(format_and_push($data, 'own_id', $optional, '', 'int', false))
		$where_str .= "`acct`.`own_id` = " . $optional['own_id']  . " and ";

	if(format_and_push($data, 'acct', $optional, '', 'string', false)) {
		$where_str .= "`acct`.`acct` = ? and ";
		array_push($values, $optional['acct']);
	}

	if(isset($data['search'])) {
		if(format_and_push($data['search'], 'value', $optional, 'search', 'string', false)) {
			$where_str .= "(`acct`.`acct` like ?) and ";
			array_push($values, "%" . $optional['search'] . "%");
		}
	}

	$order_map = array(
		"isp_id",
		"acct_id",
		"acct",
		"vlan_in",
		"vlan_out",
		"disabled",
		"update_time",
		"create_time"
	);

	$order = array();
	if(isset($optional['order'])) {
		foreach($optional['order'] as $op) {
			if(isset($op['column']) && isset($data['columns'][$op['column']]['data'])) {
				if(in_array($data['columns'][$op['column']]['data'], $order_map))
					array_push($order, "`" . $data['columns'][$op['column']]['data'] . "` " . $op['dir']);
			}
		}
	}

	if(count($order) > 0)
		$order_by = 'order by ' . implode(', ', $order);
	else
		$order_by = '';

	if($where_str != '')
		$where_str = substr("where $where_str", 0, -5);

	$keys = 'acct.`isp_id`,
ent.`name` as `isp`,
acct.`acct`';

	$sql = "select $keys from cloud_account_isp as `acct`";
	$sql.= " left join cloud_enterpise_isp as `ent`";
	$sql.= " on `ent`.`isp_id` = `acct`.`isp_id`";
	$sql.= " $where_str $order_by limit 10 ";

//	array_push($values, $optional['limit']);
//	array_push($values, $optional['offset']);

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	$result['rows'] = $rows;

	return $result;
}

function api_save($data)
{
	global $nidb;

	$filter = array(
		'acct_id'	=> new \FilterVarID(),
		'own_id'	=> new \FilterVarID(true),
	);
	if(($where = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}
	if(isset($where['acct_id']) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '帐号不能为空。');
		return false;
	}

	$filter = array(
		'isp_id'	=> new \FilterVarInt(),
		'pxyacct'	=> new \FilterVarStr(),
		'pxypass'	=> new \FilterVarStr(),
		'ippool'	=> new \FilterVarStr(),
		'pxyvlan_in'=> new \FilterVarInt(),
		'pxyvlan_out'=> new \FilterVarInt(),
	);
	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}

	if(count($frmData) <= 0) {
		set_errmsg(MSG_LEVEL_DEF, __function__, "修改已放弃！无任何内容需要保存。");
		return false;
	}

	$frmData['update_time'] = time();

	$nidb->beginTransaction();

	try {
		if(update_data('cloud_account', $frmData, $where) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			$nidb->rollBack();
			return false;
		}
	}
	catch (PDOException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		$nidb->rollBack();
		return false;
	}
	
	// 修正所有代拨密码
	$rows = false;
	if(isset($frmData['pxyacct']) && $frmData['pxyacct'] != '') {
		try {
			$whereX = array('pxyacct' => $frmData['pxyacct']);
			$dataX = array(
				'pxypass' => $frmData['pxypass']
			);
			if(update_data('cloud_account', $dataX, $whereX) === false) {
				$errmsg = implode(' ', $nidb->errorInfo());
				set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
				$nidb->rollBack();
				return false;
			}
			
			$sql = "select * from cloud_account where `disabled` < " . TB_ROW_DEL_VALUE;
			$sql.=" and `own_id` = ? and `ent_id` = ?";
			$sql.=" and `pxyacct` = ? and `acct_id` != ?";

			$sth = $nidb->prepare($sql);

			$index = 1;

			$sth->bindParam($index++, $grp->own_id, \PDO::PARAM_INT);
			$sth->bindParam($index++, $grp->ent_id, \PDO::PARAM_INT);
			$sth->bindParam($index++, $frmData['pxyacct'], \PDO::PARAM_STR);
			$sth->bindParam($index++, $where['acct_id'], \PDO::PARAM_INT);

			$sth->execute();
			$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			$nidb->rollBack();
			return false;
		}

	}

	$nidb->commit();

//	synclog_push(RA_SYNC_ACCOUNT, $where['acct_id']);
	ni_app_load('account', 'info');
	$info = \raas\apps\account\info($where, 1);
	raaseye_acct_add($info);

	if($rows) {
		foreach($rows as $row) {
			 raaseye_acct_add((array)$row);
		}
	}

	return true;
}

function remove($data)
{
	global $nidb;

	$filter = array (
		'isp_id'	=> new \FilterVarID(true),
		'acct_id'	=> new \FilterVarID(true),
		'own_id'	=> new \FilterVarID(true),
		'acct'		=> new \FilterVarStr(true),
	);
	if(($where = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}

	// is exist?
	$sql = "select * from cloud_account_isp where `isp_id` = ? and `own_id` = ? and `acct_id` = ? and `acct` = ? and `disabled` < " . TB_ROW_DEL_VALUE;

	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $where['isp_id'], \PDO::PARAM_INT);
		$sth->bindParam(2, $where['own_id'], \PDO::PARAM_INT);
		$sth->bindParam(3, $where['acct_id'], \PDO::PARAM_INT);
		$sth->bindParam(4, $where['acct'], \PDO::PARAM_STR);
		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	if(count($rows) <= 0) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '找不到此记录。');
		return false;
	}

	// have group or acctcnt ?
//	$isp = $rows[0];

	$nidb->beginTransaction();
	try {
		$sql = 'delete from cloud_account_isp where `isp_id` = ? and `own_id` = ? and `acct_id` = ? and `acct` = ?';
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $where['isp_id'], \PDO::PARAM_INT);
		$sth->bindParam(2, $where['own_id'], \PDO::PARAM_INT);
		$sth->bindParam(3, $where['acct_id'], \PDO::PARAM_INT);
		$sth->bindParam(4, $where['acct'], \PDO::PARAM_STR);
		$sth->execute();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		$nidb->rollBack();
		return false;
	}

	$nidb->commit();

	return true;
}

function info($data)
{
	global $nidb;

	$filter = array (
		'acct_id'	=> new \FilterVarID(true),
		'own_id'	=> new \FilterVarID(true),
		'acct'		=> new \FilterVarStr(true),
		'isp_id'	=> new \FilterVarID(true),
	);
	if(($where = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}

	// is exist?
	$sql = "select * from cloud_account_isp where `isp_id` = ? and `own_id` = ? and `acct_id` = ? and `acct` = ? and `disabled` < " . TB_ROW_DEL_VALUE;

	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $where['isp_id'], \PDO::PARAM_INT);
		$sth->bindParam(2, $where['own_id'], \PDO::PARAM_INT);
		$sth->bindParam(3, $where['acct_id'], \PDO::PARAM_INT);
		$sth->bindParam(4, $where['acct'], \PDO::PARAM_STR);
		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(count($rows) <= 0) 
		return false;

	return $rows[0];
}

function save($data)
{
	global $nidb;

	$filter = array(
		'isp_id'	=> new \FilterVarID(true),
		'own_id'	=> new \FilterVarID(true),
		'acct_id'	=> new \FilterVarID(true),
		'acct'		=> new \FilterVarStr(true),
	);
	if(($where = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}

	$filter = array(
//		'own_id'	=> new \FilterVarID(true),
//		'disabled'	=> new \FilterVarInt(),
		'vlan_in'	=> new \FilterVarInt(),
		'vlan_out'	=> new \FilterVarInt(),
		'pass'		=> new \FilterVarStr(),
	);
	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}

	if(count($frmData) <= 0) {
		set_errmsg(MSG_LEVEL_DEF, __function__, "修改已放弃！无任何内容需要保存。");
		return false;
	}

	$frmData['update_time'] = time();

	$nidb->beginTransaction();

	try {
		if(update_data('cloud_account_isp', $frmData, $where) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			$nidb->rollBack();
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		$nidb->rollBack();
		return false;
	}

	$nidb->commit();

	return true;
}
