<?php
namespace raas\apps\account\logger\modify;


function select($data)
{
	global $nidb;

	$values = array();
	$optional = array();
	// set order, limit, offset value
	format_list_arg($data, $optional);

	$where_str = '';
	if(format_and_push($data, 'acctlog_id', $optional, '', 'int', false))
		$where_str .= "`acctlog_id` = " . $optional['acctlog_id']  . " and ";

	if(format_and_push($data, 'startm', $optional, '', 'int', false))
		$where_str .= "`create_time` >= " . $optional['startm'] . " and ";

	if(format_and_push($data, 'endtm', $optional, '', 'int', false))
		$where_str .= "`create_time` < " . $optional['endtm']  . " and ";

	// set custom options
	if(format_and_push($data, 'own_id', $optional, '', 'int', false))
		$where_str .= "`own_id` = " . $optional['own_id']  . " and ";
	
	if(format_and_push($data, 'ent_id', $optional, '', 'int', false))
		$where_str .= "`ent_id` = " . $optional['ent_id']  . " and ";

	if(format_and_push($data, 'acct_id', $optional, '', 'int', false))
		$where_str .= "`acct_id` = " . $optional['acct_id']  . " and ";
	
	if(format_and_push($data, 'user_id', $optional, '', 'int', false))
		$where_str .= "`user_id` = " . $optional['user_id']  . " and ";
	
	if(format_and_push($data, 'account', $optional, '', 'string', false)) {
		$where_str .= "`account` = ? and ";
		array_push($values, $optional['account']);
	}
	
	if(format_and_push($data, 'username', $optional, '', 'string', false)) {
		$where_str .= "`username` = ? and ";
		array_push($values, $optional['username']);
	}

	if(format_and_push($data, 'action', $optional, '', 'string', false)) {
		$where_str .= "`action` = ? and ";
		array_push($values, $optional['action']);
	}

	if(format_and_push($data, 'remark', $optional, '', 'string', false)) {
		$where_str .= "`remark` like '%?%' and ";
		array_push($values, $optional['remark']);
	}

	if(isset($data['search'])) {
		if(format_and_push($data['search'], 'value', $optional, 'search', 'string', false)) {
			$where_str .= "(`account` like ? or `username` like ? or `remark` like ?) and ";
			array_push($values, "%" . $optional['search'] . "%");
			array_push($values, "%" . $optional['search'] . "%");
			array_push($values, "%" . $optional['search'] . "%");
		}
	}

	if($where_str != '')
		$where_str = substr("where $where_str", 0, -5);


	$order_map = array(
		"acctlog_id",
		"own_id",
		"ent_id",
		"acct_id",
		"user_id",
		"account",
		"username",
		"oper_ip",
		"action",
		"data",
		"create_time",
	);

	$order = array();
	if(isset($optional['order'])) {
		foreach($optional['order'] as $op) {
			if(isset($op['column']) && isset($data['columns'][$op['column']]['data'])) {
				if(in_array($data['columns'][$op['column']]['data'], $order_map))
					array_push($order, "`" . $data['columns'][$op['column']]['data'] . "` " . $op['dir']);
			}
		}
	}

	if(count($order) > 0)
		$order_by = 'order by ' . implode(', ', $order);
	else
		$order_by = '';

	$sql = "select count(*) as `total` from cloud_account_modify_logger $where_str";

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	if(count($rows) > 0) {
		$result = array(
			'total' => $rows[0]->total,
		);
	}
	else {
		$result = array(
			'total' => 0,
		);
	}

	$keys = '*';
	$sql = "select $keys from cloud_account_modify_logger $where_str $order_by limit ? offset ? ";

	array_push($values, $optional['limit']);
	array_push($values, $optional['offset']);

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	$result['rows'] = $rows;

	if(isset($data['draw'])) $result['draw'] = $data['draw'];

	return $result;
}

function _create($frmData)
{
	global $nidb;

	// $frmData['create_time'] = time();

	try {
		if(insert_data('cloud_account_modify_logger', $frmData) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
		$frmData['acctlog_id'] = $nidb->lastInsertId();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	return $frmData;
}

function create($data)
{
	global $nidb, $user;

	$filter = array(
		'own_id'	=> new \FilterVarID(true),
		'ent_id'	=> new \FilterVarID(true),
		'user_id'	=> new \FilterVarInt(),
		'acct_id'	=> new \FilterVarID(true),
		'account'	=> new \FilterVarStr(true),
		'username'	=> new \FilterVarStr(true),
		'action'	=> new \FilterVarStr(true),
		'data'		=> new \FilterVarUkn(),
		'remark'	=> new \FilterVarStr()
	);

	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}
	
	if(empty($frmData['action'])) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '动作不能为空！');
		return false;
	}

	ni_app_load('account', 'info');

	// find account is exist
	$acct = \raas\apps\account\info(array('acct_id' => $frmData['acct_id'], 'own_id' => $frmData['own_id']));
	if($acct === false) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '无此帐号！');
		return false;
	}
	if(isset($frmData['user_id']) && $frmData['user_id'] > 0
	&&(isset($frmData['username']) == false || $frmData['username'] == '')) {
		$user = \raas\apps\user\_info($frmData['user_id'], $frmData['own_id']);
		if($user === false) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '无效的操作员帐号！');
			return false;
		}
		if($user->disabled != 0) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '您的帐号已被冻结！无法完成此操作。');
			return false;
		}
		
		$frmData['username'] = $user->name;
	}

	$frmData['account'] = $acct->account;
	
	if(isset($data['data']) && is_array($data['data']))
		$frmData['data'] = json_encode($data['data']);
	else
		$frmData['data'] = '';


	$frmData['oper_ip'] = get_real_ip();
	$frmData['create_time'] = time();

	$frmData = _create($frmData);
	
	return $frmData;
}
