<?php
namespace raas\apps\enterpise\isp;


function _create($frmData)
{
	global $nidb;


	$frmData['update_time'] = time();
	$frmData['create_time'] = $frmData['update_time'];

	try {
		if(insert_data('cloud_enterpise_isp', $frmData) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
		$frmData['isp_id'] = $nidb->lastInsertId();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return $frmData;
}

function create($data)
{
	global $nidb;

	$filter = array(
		'ent_id'	=> new \FilterVarID(),
		'group_id'	=> new \FilterVarInt(),
		'own_id'	=> new \FilterVarID(true),
		'name'		=> new \FilterVarStr(true),
//		'disabled'	=> new \FilterVarInt(),
		'vlan_in'	=> new \FilterVarInt(),
		'vlan_out'	=> new \FilterVarInt(),
		'ippool'	=> new \FilterVarStr(),
		'pxy_type'	=> new \FilterVarInt(),
		'serip'		=> new \FilterVarStr(),
		'nas'		=> new \FilterVarStr(),
		'secret'	=> new \FilterVarStr(),
		'lport'		=> new \FilterVarInt(),
		'access'	=> new \FilterVarInt(),
		'accounting'=> new \FilterVarInt(),
		'coa'		=> new \FilterVarInt(),
		'remark'	=> new \FilterVarStr(),
	);
	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}
	if(empty($frmData['name'])) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '运营商名称不能为空！');
		return false;
	}

	$nidb->beginTransaction();

	if(($frmData = _create($frmData)) === false) {
		$nidb->rollBack();
		return false;
	}

	$nidb->commit();

	raaseye_enterpise_isp_add($frmData);
	
	return $frmData;
}

function select($data)
{
	global $nidb;
	global $user;

	$values = array();
	$optional = array();
	// set order, limit, offset value
	format_list_arg($data, $optional);

	$where_str = '';
	if(format_and_push($data, 'disabled', $optional, '', 'int', false))
		$where_str .= "`disabled` = " . $optional['disabled'] . " and ";
	else
		$where_str .= "`disabled` < " . TB_ROW_DEL_VALUE . " and ";

	if(format_and_push($data, 'upstartm', $optional, '', 'int', false))
		$where_str .= "`update_time` >= " . $optional['upstartm'] . " and ";

	if(format_and_push($data, 'endtm', $optional, '', 'int', false)) 
		$where_str .= "`update_time` < " . $optional['upendtm']  . " and ";

	if(format_and_push($data, 'startm', $optional, '', 'int', false))
		$where_str .= "`create_time` >= " . $optional['startm'] . " and ";

	if(format_and_push($data, 'endtm', $optional, '', 'int', false))
		$where_str .= "`create_time` < " . $optional['endtm']  . " and ";

	// set custom options
	if(format_and_push($data, 'ent_id', $optional, '', 'int', false))
		$where_str .= "`ent_id` = " . $optional['ent_id']  . " and ";

	if(format_and_push($data, 'isp_id', $optional, '', 'int', false))
		$where_str .= "`isp_id` = " . $optional['isp_id']  . " and ";

	if(format_and_push($data, 'ids', $optional, '', 'string', false)) {
		$idsstr = explode(',', $optional['ids']);
		$ids = array();
		foreach($idsstr as $val) {
			$val = intval($val);
			if($val > 0)
				array_push($ids, $val);
		}
		if(count($ids) > 0)
			$where_str .= "`isp_id` in (" . implode(',', $ids) . ") and ";
	}
	
	if(format_and_push($data, 'own_id', $optional, '', 'int', false))
		$where_str .= "`own_id` = " . $optional['own_id']  . " and ";

	if(format_and_push($data, 'name', $optional, '', 'string', false)) {
		$where_str .= "`name` = ? and ";
		array_push($values, $optional['name']);
	}

	if(isset($data['search'])) {
		if(format_and_push($data['search'], 'value', $optional, 'search', 'string', false)) {
			$where_str .= "(`name` like ? or `remark` like ? or `nas` like ? or `serip` like ?) and ";
			array_push($values, "%" . $optional['search'] . "%");
			array_push($values, "%" . $optional['search'] . "%");
			array_push($values, "%" . $optional['search'] . "%");
			array_push($values, "%" . $optional['search'] . "%");
		}
	}

	$order_map = array(
		"isp_id",
		"ent_id",
		"own_id",
		"group_id",
		"name",
		"pxy_type",
		"serip",
		"nas",
		"lport",
		"secret",
		"access",
		"accounting",
		"coa",
		"ippool",
		"vlan_in",
		"vlan_out",
		"disabled",
		"update_time",
		"create_time"
	);

	$order = array();
	if(isset($optional['order'])) {
		foreach($optional['order'] as $op) {
			if(isset($op['column']) && isset($data['columns'][$op['column']]['data'])) {
				if(in_array($data['columns'][$op['column']]['data'], $order_map))
					array_push($order, "`" . $data['columns'][$op['column']]['data'] . "` " . $op['dir']);
			}
		}
	}

	if(count($order) > 0)
		$order_by = 'order by ' . implode(', ', $order);
	else
		$order_by = '';
/*
	// 限制设置了负责企业的操作员只能访问其负责的企业
	if(gettype($user) == 'object' && $user->entcnt > 0) {
		$sql = "select group_concat(`ent_id`) as `ids` from cloud_user_group_list where `user_id` = ? group by `ent_id`";
		try {
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $user->user_id, \PDO::PARAM_INT);
			if($sth->execute())
				$row = $sth->fetch(\PDO::FETCH_ASSOC);
			else
				$row = false;
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}

		if($row && empty($row['ids']) == false) {
			$where_str .= '(`ent_id` in (' . $row['ids'] . ') or `admin` = ?) and ';
			array_push($values, $user->account);
		}
	}
*/
	if($where_str != '')
		$where_str = substr("where $where_str", 0, -5);

	$sql = "select count(*) as `total` from cloud_enterpise_isp $where_str";
	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);

		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	if(count($rows) > 0) {
		$result = array(
			'total' => $rows[0]->total,
		);
	}
	else {
		$result = array(
			'total' => 0,
		);
	}

	$sql = "select * from cloud_enterpise_isp $where_str $order_by limit ? offset ? ";

	array_push($values, $optional['limit']);
	array_push($values, $optional['offset']);

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	$result['rows'] = $rows;
//	$result['sql'] = $sql;
//	$result['values'] = $values;
//  select `ent_id`, `name`, `domain`, `iwancnt`, `iwacctcnt`, `province`, `city`, `district`, `address`, `remark` from cloud_enterpise_isp where `disabled` < 120 and `iwansvr` = 1 order by `acctcnt` desc limit 20 offset 0 

	if(isset($data['draw'])) $result['draw'] = $data['draw'];

	return $result;
}

function remove($data)
{
	global $nidb;

	$filter = array(
		'isp_id'	=> new \FilterVarID(true),
		'own_id'	=> new \FilterVarID(true),
	);
	if(($where = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}

	// is exist?
	$sql = "select * from cloud_enterpise_isp where `isp_id` = ? and `disabled` < " . TB_ROW_DEL_VALUE;

	try {
		$sth = $nidb->prepare($sql);

		$index = 1;
		$sth->bindParam($index++, $where['isp_id'], \PDO::PARAM_INT);

		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	if(count($rows) <= 0) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '找不到此记录。');
		return false;
	}

	// have group or acctcnt ?
	$isp = $rows[0];
	
//	$frmData = array();
//	$frmData['disabled'] = TB_ROW_DEL_VALUE;
//	$frmData['update_time'] = time();

	$nidb->beginTransaction();
	try {
/*
		if(update_data('cloud_enterpise_isp', $frmData, $where) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			$nidb->rollBack();
			return false;
		}
*/
		$sql = 'delete from cloud_enterpise_isp where `isp_id` = ? and `own_id` = ?';
		$sth = $nidb->prepare($sql);

		$sth->bindParam(1, $where['isp_id'], \PDO::PARAM_INT);
		$sth->bindParam(2, $where['own_id'], \PDO::PARAM_INT);
		$sth->execute();
		
		
		// 删除企业下的用户代理帐号
		$sql = 'delete from cloud_account_isp where `isp_id` = ?';
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $where['isp_id'], \PDO::PARAM_INT);
		$sth->execute();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		$nidb->rollBack();
		return false;
	}

	$nidb->commit();

	raaseye_enterpise_isp_set($where);

	return true;
}

function info($data)
{
	global $nidb;

	$filter = array (
		'own_id'	=> new \FilterVarID(true),
		'isp_id'	=> new \FilterVarID(true),
	);
	if(($where = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}

	// is exist?
	$sql = "select * from cloud_enterpise_isp where `isp_id` = ? and `own_id` = ? and `disabled` < " . TB_ROW_DEL_VALUE;

	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $where['isp_id'], \PDO::PARAM_INT);
		$sth->bindParam(2, $where['own_id'], \PDO::PARAM_INT);
		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(count($rows) <= 0) 
		return false;

	return $rows[0];
}

function save($data)
{
	global $nidb;

	$filter = array(
		'isp_id'	=> new \FilterVarID(true),
		'own_id'	=> new \FilterVarID(true),
	);
	if(($where = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}

	$filter = array(
		'ent_id'	=> new \FilterVarID(),
		'group_id'	=> new \FilterVarInt(),
//		'own_id'	=> new \FilterVarID(true),
		'name'		=> new \FilterVarStr(true),
//		'disabled'	=> new \FilterVarInt(),
		'vlan_in'	=> new \FilterVarInt(),
		'vlan_out'	=> new \FilterVarInt(),
		'ippool'	=> new \FilterVarStr(),		
		'pxy_type'	=> new \FilterVarInt(),
		'serip'		=> new \FilterVarStr(),
		'nas'		=> new \FilterVarStr(),
		'secret'	=> new \FilterVarStr(),
		'lport'		=> new \FilterVarInt(),
		'access'	=> new \FilterVarInt(),
		'accounting'=> new \FilterVarInt(),
		'coa'		=> new \FilterVarInt(),
		'remark'	=> new \FilterVarStr(),
	);
	if(($frmData = filter_data($data, $filter, $errmsg)) === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, $errmsg);
		return false;
	}

	if(count($frmData) <= 0) {
		set_errmsg(MSG_LEVEL_DEF, __function__, "修改已放弃！无任何内容需要保存。");
		return false;
	}

	$frmData['update_time'] = time();

	$nidb->beginTransaction();

	try {
		if(update_data('cloud_enterpise_isp', $frmData, $where) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			$nidb->rollBack();
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		$nidb->rollBack();
		return false;
	}

	$nidb->commit();

	// is exist?
	$sql = "select * from cloud_enterpise_isp where `isp_id` = ? and `own_id` = ? and `disabled` < " . TB_ROW_DEL_VALUE;

	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $where['isp_id'], \PDO::PARAM_INT);
		$sth->bindParam(2, $where['own_id'], \PDO::PARAM_INT);
		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return true;
	}

	if(count($rows) <= 0) 
		return true;

	raaseye_enterpise_isp_set((array)$rows[0]);
	
	return true;
}
