<?PHP

function uuid()
{
     // Generate 128 bit random sequence 
     $randmax_bits = strlen(base_convert(mt_getrandmax(), 10, 2));  // how many bits is mt_getrandmax()
     $x = '';
     while (strlen($x) < 128) {
         $maxbits = (128 - strlen($x) < $randmax_bits) ? 128 - strlen($x) :  $randmax_bits;
         $x .= str_pad(base_convert(mt_rand(0, pow(2,$maxbits)), 10, 2), $maxbits, "0", STR_PAD_LEFT);
     }

     // break into fields
     $a = array();
     $a['time_low_part'] = substr($x, 0, 32);
     $a['time_mid'] = substr($x, 32, 16);
     $a['time_hi_and_version'] = substr($x, 48, 16);
     $a['clock_seq'] = substr($x, 64, 16);
     $a['node_part'] =  substr($x, 80, 48);
     
     // Apply bit masks for "random or pseudo-random" version per RFC
     $a['time_hi_and_version'] = substr_replace($a['time_hi_and_version'], '0100', 0, 4);
     $a['clock_seq'] = substr_replace($a['clock_seq'], '10', 0, 2);

    // Format output
    return sprintf('%s%s%s%s%s',
        str_pad(base_convert($a['time_low_part'], 2, 16), 8, "0", STR_PAD_LEFT),
        str_pad(base_convert($a['time_mid'], 2, 16), 4, "0", STR_PAD_LEFT),
        str_pad(base_convert($a['time_hi_and_version'], 2, 16), 4, "0", STR_PAD_LEFT),
        str_pad(base_convert($a['clock_seq'], 2, 16), 4, "0", STR_PAD_LEFT),
        str_pad(base_convert($a['node_part'], 2, 16), 12, "0", STR_PAD_LEFT));

}

function format_and_push($in_data, $name, &$out_data, string $alias = '', string $type = 'string', $can_be_empty = true)
{
	if(isset($in_data[$name]) == false) return false;

	if(gettype($in_data[$name]) == 'string')
		$val = trim($in_data[$name]);
	else 
		$val = $in_data[$name];

	if($can_be_empty !== true && empty($val)) return false;

	switch ($type) {
		case 'integer':
		case 'int':
			$val = intval($val);
			break;
		case 'double':
		case 'float':
			$val = floatval($val);
			break;
		case 'array':
			$val = (array)$val;
			break;
		case 'object':
			$val = (object)$val;
			break;
		default:
			break;
	}

	if($alias != '')
		$out_data[$alias] = $val;
	else
		$out_data[$name] = $val;

	return true;
}

function format_list_arg(array $in_data, array &$out_data, string $def_orderdir = 'desc', int $def_limit = 20)
{
	if(format_and_push($in_data, 'order', $out_data, '', 'array', false) == false) 
		$out_data['order'] = array();
	else {
		foreach($out_data['order'] as $k => $op) {
			if(in_array($op['dir'], array('asc', 'desc')) == false)
				$out_data['order'][$k]['dir'] = 'desc';
		}
	}

	if(format_and_push($in_data, 'limit', $out_data, '', 'int', false) == false) 
		$out_data['limit'] = $def_limit;
	else
	if($out_data['limit'] > 200)
		$out_data['limit'] = 200;
	else
	if($out_data['limit'] <= 0)
		$out_data['limit'] = $def_limit;
	format_and_push($in_data, 'length', $out_data, 'limit', 'int', false);

	if(format_and_push($in_data, 'offset', $out_data, '', 'int', false) == false) 
		$out_data['offset'] = 0;

	format_and_push($in_data, 'start', $out_data, 'offset', 'int', false);
	if(format_and_push($in_data, 'page', $out_data, 'offset', 'int', false)) {
		$out_data['offset']--;
		if($out_data['offset'] >= 0)
			$out_data['offset'] = $out_data['offset'] * $out_data['limit'];
		else
			$out_data['offset'] = 0;
	}
		
}

function get_real_ip()
{
	$ip = false;

	if(isset($_REQUEST["_src_client_ip"]) && !empty($_REQUEST["_src_client_ip"])) {
		return $_REQUEST["_src_client_ip"];
	}
	
	if(!empty($_SERVER["HTTP_CLIENT_IP"]))
		$ip = $_SERVER["HTTP_CLIENT_IP"];

	if (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {

		$ips = explode(", ", $_SERVER['HTTP_X_FORWARDED_FOR']);
		if ($ip) {
			array_unshift($ips, $ip);
			$ip = false;
		}

		for ($i = 0; $i < count($ips); $i++) {
			if (!eregi("^(10|172\.16|192\.168)\.", $ips[$i])) {
				$ip = $ips[$i];
				break;
			}
		}

	}

	return ($ip ? $ip : (isset($_SERVER['REMOTE_ADDR']) ? $_SERVER['REMOTE_ADDR'] : '127.0.0.1'));
}

function is_ipaddr($ip)
{
	return (long2ip(ip2long($ip)) === trim($ip));
}

